@@ -71,19 +71,31 @@ start_server {tags {"latency-monitor needs:latency"}} {
         assert {[string length [r latency histogram blabla set get]] > 0}
     }
 
+tags {"needs:debug"} {
+    set old_threshold_value [lindex [r config get latency-monitor-threshold] 1]
+
     test {Test latency events logging} {
+        r config set latency-monitor-threshold 200
+        r latency reset
         r debug sleep 0.3
         after 1100
         r debug sleep 0.4
         after 1100
         r debug sleep 0.5
+        r config set latency-monitor-threshold 0
         assert {[r latency history command] >= 3}
-    } {} {needs:debug}
+    }
 
     test {LATENCY HISTORY output is ok} {
+        set res [r latency history command]
+        if {$::verbose} {
+            puts "LATENCY HISTORY data:"
+            puts $res
+        }
+
         set min 250
         set max 450
-        foreach event [r latency history command] {
+        foreach event $res {
             lassign $event time latency
             if {!$::no_latency} {
                 assert {$latency >= $min && $latency <= $max}
@@ -95,7 +107,13 @@ start_server {tags {"latency-monitor needs:latency"}} {
     }
 
     test {LATENCY LATEST output is ok} {
-        foreach event [r latency latest] {
+        set res [r latency latest]
+        if {$::verbose} {
+            puts "LATENCY LATEST data:"
+            puts $res
+        }
+
+        foreach event $res {
             lassign $event eventname time latency max
             assert {$eventname eq "command"}
             if {!$::no_latency} {
@@ -106,6 +124,24 @@ start_server {tags {"latency-monitor needs:latency"}} {
         }
     }
 
+    test {LATENCY GRAPH can output the event graph} {
+        set res [r latency graph command]
+        if {$::verbose} {
+            puts "LATENCY GRAPH data:"
+            puts $res
+        }
+        assert_match {*command*high*low*} $res
+
+        # These numbers are taken from the "Test latency events logging" test.
+        # (debug sleep 0.3) and (debug sleep 0.5), using range to prevent timing issue.
+        regexp "command - high (.*?) ms, low (.*?) ms" $res -> high low
+        assert_morethan_equal $high 500
+        assert_morethan_equal $low 300
+    }
+
+    r config set latency-monitor-threshold $old_threshold_value
+} ;# tag
+
     test {LATENCY of expire events are correctly collected} {
         r config set latency-monitor-threshold 20
         r flushdb
@@ -124,6 +160,12 @@ start_server {tags {"latency-monitor needs:latency"}} {
             fail "key wasn't expired"
         }
         assert_match {*expire-cycle*} [r latency latest]
+
+        test {LATENCY GRAPH can output the expire event graph} {
+             assert_match {*expire-cycle*high*low*} [r latency graph expire-cycle]
+        }
+
+        r config set latency-monitor-threshold 200
     }
 
     test {LATENCY HISTORY / RESET with wrong event name is fine} {
